@@ -1,27 +1,26 @@
 /* diff3 - compare three files line by line
 
-   Copyright (C) 1988, 1989, 1992, 1993, 1994, 1995, 1996, 1998, 2001,
-   2002 Free Software Foundation, Inc.
+   Copyright (C) 1988-1989, 1992-1996, 1998, 2001-2002, 2004, 2006, 2009-2011
+   Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-   See the GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; see the file COPYING.
-   If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "system.h"
 #include "paths.h"
 
-static char const authorship_msgid[] = N_("Written by Randy Smith.");
+#include <stdio.h>
+#include <unlocked-io.h>
 
 #include <c-stack.h>
 #include <cmpbuf.h>
@@ -29,11 +28,17 @@ static char const authorship_msgid[] = N_("Written by Randy Smith.");
 #include <exitfail.h>
 #include <file-type.h>
 #include <getopt.h>
-#include <inttostr.h>
-#include <quotesys.h>
-#include <stdio.h>
+#include <progname.h>
+#include <sh-quote.h>
 #include <version-etc.h>
 #include <xalloc.h>
+#include <xfreopen.h>
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "diff3"
+
+#define AUTHORS \
+  proper_name ("Randy Smith")
 
 /* Internal data structures and macros for the diff3 program; includes
    data structures for both diff3 diffs and normal diffs.  */
@@ -130,6 +135,9 @@ struct diff3_block {
 /* If nonzero, treat all files as text files, never as binary.  */
 static bool text;
 
+/* Remove trailing carriage returns from input.  */
+static bool strip_trailing_cr;
+
 /* If nonzero, write out an ed script instead of the standard diff3 format.  */
 static bool edscript;
 
@@ -157,8 +165,6 @@ static bool finalwrite;
 /* If nonzero, output a merged file.  */
 static bool merge;
 
-char *program_name;
-
 static char *read_diff (char const *, char const *, char **);
 static char *scan_diff_line (char *, char **, size_t *, char *, char);
 static enum diff_type process_diff_control (char **, struct diff_block *);
@@ -184,23 +190,25 @@ static char const *diff_program = DEFAULT_DIFF_PROGRAM;
 enum
 {
   DIFF_PROGRAM_OPTION = CHAR_MAX + 1,
-  HELP_OPTION
+  HELP_OPTION,
+  STRIP_TRAILING_CR_OPTION
 };
 
 static struct option const longopts[] =
 {
-  {"text", 0, 0, 'a'},
-  {"show-all", 0, 0, 'A'},
-  {"ed", 0, 0, 'e'},
   {"diff-program", 1, 0, DIFF_PROGRAM_OPTION},
-  {"show-overlap", 0, 0, 'E'},
+  {"easy-only", 0, 0, '3'},
+  {"ed", 0, 0, 'e'},
+  {"help", 0, 0, HELP_OPTION},
+  {"initial-tab", 0, 0, 'T'},
   {"label", 1, 0, 'L'},
   {"merge", 0, 0, 'm'},
-  {"initial-tab", 0, 0, 'T'},
   {"overlap-only", 0, 0, 'x'},
-  {"easy-only", 0, 0, '3'},
+  {"show-all", 0, 0, 'A'},
+  {"show-overlap", 0, 0, 'E'},
+  {"strip-trailing-cr", 0, 0, STRIP_TRAILING_CR_OPTION},
+  {"text", 0, 0, 'a'},
   {"version", 0, 0, 'v'},
-  {"help", 0, 0, HELP_OPTION},
   {0, 0, 0, 0}
 };
 
@@ -221,54 +229,58 @@ main (int argc, char **argv)
   char **file;
   struct stat statb;
 
-  exit_failure = 2;
+  exit_failure = EXIT_TROUBLE;
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
-  c_stack_action (argv, 0);
+  c_stack_action (0);
 
   while ((c = getopt_long (argc, argv, "aeimvx3AEL:TX", longopts, 0)) != -1)
     {
       switch (c)
 	{
 	case 'a':
-	  text = 1;
+	  text = true;
 	  break;
 	case 'A':
-	  show_2nd = 1;
-	  flagging = 1;
+	  show_2nd = true;
+	  flagging = true;
 	  incompat++;
 	  break;
 	case 'x':
-	  overlap_only = 1;
+	  overlap_only = true;
 	  incompat++;
 	  break;
 	case '3':
-	  simple_only = 1;
+	  simple_only = true;
 	  incompat++;
 	  break;
 	case 'i':
-	  finalwrite = 1;
+	  finalwrite = true;
 	  break;
 	case 'm':
-	  merge = 1;
+	  merge = true;
 	  break;
 	case 'X':
-	  overlap_only = 1;
+	  overlap_only = true;
 	  /* Fall through.  */
 	case 'E':
-	  flagging = 1;
+	  flagging = true;
 	  /* Fall through.  */
 	case 'e':
 	  incompat++;
 	  break;
 	case 'T':
-	  initial_tab = 1;
+	  initial_tab = true;
+	  break;
+	case STRIP_TRAILING_CR_OPTION:
+	  strip_trailing_cr = true;
 	  break;
 	case 'v':
-	  version_etc ("diff3", authorship_msgid);
+	  version_etc (stdout, PROGRAM_NAME, PACKAGE_NAME, Version,
+		       AUTHORS, (char *) NULL);
 	  check_stdout ();
 	  return EXIT_SUCCESS;
 	case DIFF_PROGRAM_OPTION:
@@ -331,13 +343,13 @@ main (int argc, char **argv)
 
   common = 2 - (edscript | merge);
 
-  if (strcmp (file[common], "-") == 0)
+  if (STREQ (file[common], "-"))
     {
       /* Sigh.  We've got standard input as the common file.  We can't
 	 call diff twice on stdin.  Use the other arg as the common
 	 file instead.  */
       common = 3 - common;
-      if (strcmp (file[0], "-") == 0 || strcmp (file[common], "-") == 0)
+      if (STREQ (file[0], "-") || STREQ (file[common], "-"))
 	fatal ("`-' specified for more than one input file");
     }
 
@@ -349,7 +361,7 @@ main (int argc, char **argv)
     rev_mapping[mapping[i]] = i;
 
   for (i = 0; i < 3; i++)
-    if (strcmp (file[i], "-") != 0)
+    if (! STREQ (file[i], "-"))
       {
 	if (stat (file[i], &statb) < 0)
 	  perror_with_exit (file[i]);
@@ -375,8 +387,7 @@ main (int argc, char **argv)
 			       tag_strings[0], tag_strings[1], tag_strings[2]);
   else if (merge)
     {
-      if (! freopen (file[rev_mapping[FILE0]], "r", stdin))
-	perror_with_exit (file[rev_mapping[FILE0]]);
+      xfreopen (file[rev_mapping[FILE0]], "r", stdin);
       conflicts_found
 	= output_diff3_merge (stdin, stdout, diff3, mapping, rev_mapping,
 			      tag_strings[0], tag_strings[1], tag_strings[2]);
@@ -386,7 +397,7 @@ main (int argc, char **argv)
   else
     {
       output_diff3 (stdout, diff3, mapping, rev_mapping);
-      conflicts_found = 0;
+      conflicts_found = false;
     }
 
   check_stdout ();
@@ -414,22 +425,28 @@ check_stdout (void)
 }
 
 static char const * const option_help_msgid[] = {
-  N_("-e  --ed  Output unmerged changes from OLDFILE to YOURFILE into MYFILE."),
-  N_("-E  --show-overlap  Output unmerged changes, bracketing conflicts."),
-  N_("-A  --show-all  Output all changes, bracketing conflicts."),
-  N_("-x  --overlap-only  Output overlapping changes."),
-  N_("-X  Output overlapping changes, bracketing them."),
-  N_("-3  --easy-only  Output unmerged nonoverlapping changes."),
+  N_("-A, --show-all              output all changes, bracketing conflicts"),
   "",
-  N_("-m  --merge  Output merged file instead of ed script (default -A)."),
-  N_("-L LABEL  --label=LABEL  Use LABEL instead of file name."),
-  N_("-i  Append `w' and `q' commands to ed scripts."),
-  N_("-a  --text  Treat all files as text."),
-  N_("-T  --initial-tab  Make tabs line up by prepending a tab."),
-  N_("--diff-program=PROGRAM  Use PROGRAM to compare files."),
+  N_("-e, --ed                    output ed script incorporating changes\n"
+     "                                from OLDFILE to YOURFILE into MYFILE"),
+  N_("-E, --show-overlap          like -e, but bracket conflicts"),
+  N_("-3, --easy-only             like -e, but incorporate only nonoverlapping changes"),
+  N_("-x, --overlap-only          like -e, but incorporate only overlapping changes"),
+  N_("-X                          like -x, but bracket conflicts"),
+  N_("-i                          append `w' and `q' commands to ed scripts"),
   "",
-  N_("-v  --version  Output version info."),
-  N_("--help  Output this help."),
+  N_("-m, --merge                 output actual merged file, according to\n"
+     "                                -A if no other options are given"),
+  "",
+  N_("-a, --text                  treat all files as text"),
+  N_("    --strip-trailing-cr     strip trailing carriage return on input"),
+  N_("-T, --initial-tab           make tabs line up by prepending a tab"),
+  N_("    --diff-program=PROGRAM  use PROGRAM to compare files"),
+  N_("-L, --label=LABEL           use LABEL instead of file name\n"
+     "                                (can be repeated up to three times)"),
+  "",
+  N_("    --help                  display this help and exit"),
+  N_("-v, --version               output version information and exit"),
   0
 };
 
@@ -441,14 +458,29 @@ usage (void)
   printf (_("Usage: %s [OPTION]... MYFILE OLDFILE YOURFILE\n"),
 	  program_name);
   printf ("%s\n\n", _("Compare three files line by line."));
+
+  fputs (_("\
+Mandatory arguments to long options are mandatory for short options too.\n\
+"), stdout);
   for (p = option_help_msgid;  *p;  p++)
     if (**p)
       printf ("  %s\n", _(*p));
     else
       putchar ('\n');
-  printf ("\n%s\n\n%s\n",
+  fputs (_("\n\
+The default output format is a somewhat human-readable representation of\n\
+the changes.\n\
+\n\
+The -e, -E, -x, -X (and corresponding long) options cause an ed script\n\
+to be output instead of the default.\n\
+\n\
+Finally, the -m (--merge) option causes diff3 to do the merge internally\n\
+and output the actual merged file.  For unusual input, this is more\n\
+robust than using ed.\n"), stdout);
+  printf ("\n%s\n%s\n",
 	  _("If a FILE is `-', read standard input."),
-	  _("Report bugs to <bug-gnu-utils@gnu.org>."));
+	  _("Exit status is 0 if successful, 1 if conflicts, 2 if trouble."));
+  emit_bug_reporting_address ();
 }
 
 /* Combine the two diffs together into one.
@@ -624,7 +656,6 @@ make_3way_diff (struct diff_block *thread0, struct diff_block *thread1)
 	  if (high_water_mark < D_HIGHLINE (other_diff, FC))
 	    {
 	      high_water_thread ^= 1;
-	      high_water_diff = other_diff;
 	      high_water_mark = D_HIGHLINE (other_diff, FC);
 	    }
 
@@ -787,8 +818,8 @@ using_to_diff3_block (struct diff_block *using[2],
 
 /* Copy pointers from a list of strings to a different list of
    strings.  If a spot in the second list is already filled, make sure
-   that it is filled with the same string; if not, return 0, the copy
-   incomplete.  Upon successful completion of the copy, return 1.  */
+   that it is filled with the same string; if not, return false, the copy
+   incomplete.  Upon successful completion of the copy, return true.  */
 
 static bool
 copy_stringlist (char * const fromptrs[], size_t const fromlengths[],
@@ -805,7 +836,7 @@ copy_stringlist (char * const fromptrs[], size_t const fromlengths[],
       if (*t)
 	{
 	  if (*fl != *tl || memcmp (*f, *t, *fl) != 0)
-	    return 0;
+	    return false;
 	}
       else
 	{
@@ -816,7 +847,7 @@ copy_stringlist (char * const fromptrs[], size_t const fromlengths[],
       t++; f++; tl++; fl++;
     }
 
-  return 1;
+  return true;
 }
 
 /* Create a diff3_block, with ranges as specified in the arguments.
@@ -899,8 +930,8 @@ compare_line_list (char * const list1[], size_t const lengths1[],
   while (nl--)
     if (!*l1 || !*l2 || *lgths1 != *lgths2++
 	|| memcmp (*l1++, *l2++, *lgths1++) != 0)
-      return 0;
-  return 1;
+      return false;
+  return true;
 }
 
 /* Input and parse two way diffs.  */
@@ -915,15 +946,15 @@ process_diff (char const *filea,
   char *scan_diff;
   enum diff_type dt;
   lin i;
-  struct diff_block *block_list, **block_list_end, *bptr;
+  struct diff_block *block_list;
+  struct diff_block **block_list_end = &block_list;
+  struct diff_block *bptr IF_LINT (= NULL);
   size_t too_many_lines = (PTRDIFF_MAX
 			   / MIN (sizeof *bptr->lines[1],
 				  sizeof *bptr->lengths[1]));
 
   diff_limit = read_diff (filea, fileb, &diff_contents);
   scan_diff = diff_contents;
-  block_list_end = &block_list;
-  bptr = 0; /* Pacify `gcc -W'.  */
 
   while (scan_diff < diff_limit)
     {
@@ -1007,7 +1038,7 @@ process_diff (char const *filea,
       block_list_end = &bptr->next;
     }
 
-  *block_list_end = 0;
+  *block_list_end = NULL;
   *last_block = bptr;
   return block_list;
 }
@@ -1127,13 +1158,13 @@ read_diff (char const *filea,
 {
   char *diff_result;
   size_t current_chunk_size, total;
-  int fd, wstatus;
+  int fd, wstatus, status;
   int werrno = 0;
   struct stat pipestat;
 
-#if HAVE_WORKING_FORK || HAVE_WORKING_VFORK
+#if HAVE_WORKING_FORK
 
-  char const *argv[8];
+  char const *argv[9];
   char const **ap;
   int fds[2];
   pid_t pid;
@@ -1142,6 +1173,8 @@ read_diff (char const *filea,
   *ap++ = diff_program;
   if (text)
     *ap++ = "-a";
+  if (strip_trailing_cr)
+    *ap++ = "--strip-trailing-cr";
   *ap++ = "--horizon-lines=100";
   *ap++ = "--";
   *ap++ = filea;
@@ -1151,7 +1184,7 @@ read_diff (char const *filea,
   if (pipe (fds) != 0)
     perror_with_exit ("pipe");
 
-  pid = vfork ();
+  pid = fork ();
   if (pid == 0)
     {
       /* Child */
@@ -1166,7 +1199,7 @@ read_diff (char const *filea,
 	 hosts with a nonstandard prototype for execvp.  */
       execvp (diff_program, (char **) argv);
 
-      _exit (127);
+      _exit (errno == ENOENT ? 127 : 126);
     }
 
   if (pid == -1)
@@ -1178,18 +1211,30 @@ read_diff (char const *filea,
 #else
 
   FILE *fpipe;
-  char const args[] = " -a --horizon-lines=100 -- ";
-  char *command = xmalloc (quote_system_arg (0, diff_program)
+  char const args[] = " --horizon-lines=100 -- ";
+  char *command = xmalloc (shell_quote_length (diff_program)
+			   + sizeof "-a"
+			   + sizeof "--strip-trailing-cr"
 			   + sizeof args - 1
-			   + quote_system_arg (0, filea) + 1
-			   + quote_system_arg (0, fileb) + 1);
+			   + shell_quote_length (filea) + 1
+			   + shell_quote_length (fileb) + 1);
   char *p = command;
-  p += quote_system_arg (p, diff_program);
-  strcpy (p, args + (text ? 0 : 3));
-  p += strlen (p);
-  p += quote_system_arg (p, filea);
+  p = shell_quote_copy (p, diff_program);
+  if (text)
+    {
+      strcpy (p, " -a");
+      p += 3;
+    }
+  if (strip_trailing_cr)
+    {
+      strcpy (p, " --strip-trailing-cr");
+      p += 20;
+    }
+  strcpy (p, args);
+  p += sizeof args - 1;
+  p = shell_quote_copy (p, filea);
   *p++ = ' ';
-  p += quote_system_arg (p, fileb);
+  p = shell_quote_copy (p, fileb);
   *p = 0;
   errno = 0;
   fpipe = popen (command, "r");
@@ -1228,7 +1273,7 @@ read_diff (char const *filea,
 
   *output_placement = diff_result;
 
-#if ! (HAVE_WORKING_FORK || HAVE_WORKING_VFORK)
+#if ! HAVE_WORKING_FORK
 
   wstatus = pclose (fpipe);
   if (wstatus == -1)
@@ -1243,12 +1288,18 @@ read_diff (char const *filea,
 
 #endif
 
-  if (werrno || ! (WIFEXITED (wstatus) && WEXITSTATUS (wstatus) < 2))
+  status = ! werrno && WIFEXITED (wstatus) ? WEXITSTATUS (wstatus) : INT_MAX;
+
+  if (EXIT_TROUBLE <= status)
     error (EXIT_TROUBLE, werrno,
-	   _(! werrno && WIFEXITED (wstatus) && WEXITSTATUS (wstatus) == 127
+	   _(status == 126
+	     ? "subsidiary program `%s' could not be invoked"
+	     : status == 127
 	     ? "subsidiary program `%s' not found"
-	     : "subsidiary program `%s' failed"),
-	   diff_program);
+	     : status == INT_MAX
+	     ? "subsidiary program `%s' failed"
+	     : "subsidiary program `%s' failed (exit status %d)"),
+	   diff_program, status);
 
   return diff_result + total;
 }
@@ -1351,8 +1402,8 @@ output_diff3 (FILE *outputfile, struct diff3_block *diff,
 	  int realfile = mapping[i];
 	  lin lowt = D_LOWLINE (ptr, realfile);
 	  lin hight = D_HIGHLINE (ptr, realfile);
-	  long llowt = lowt;
-	  long lhight = hight;
+	  long int llowt = lowt;
+	  long int lhight = hight;
 
 	  fprintf (outputfile, "%d:", i + 1);
 	  switch (lowt - hight)
@@ -1375,7 +1426,7 @@ output_diff3 (FILE *outputfile, struct diff3_block *diff,
 	      line = 0;
 	      do
 		{
-		  fprintf (outputfile, line_prefix);
+		  fputs (line_prefix, outputfile);
 		  cp = D_RELNUM (ptr, realfile, line);
 		  length = D_RELLEN (ptr, realfile, line);
 		  fwrite (cp, sizeof (char), length, outputfile);
@@ -1397,7 +1448,7 @@ static bool
 dotlines (FILE *outputfile, struct diff3_block *b, int filenum)
 {
   lin i;
-  bool leading_dot = 0;
+  bool leading_dot = false;
 
   for (i = 0;
        i < D_NUMLINES (b, filenum);
@@ -1406,8 +1457,8 @@ dotlines (FILE *outputfile, struct diff3_block *b, int filenum)
       char *line = D_RELNUM (b, filenum, i);
       if (line[0] == '.')
 	{
-	  leading_dot = 1;
-	  fprintf (outputfile, ".");
+	  leading_dot = true;
+	  fputc ('.', outputfile);
 	}
       fwrite (line, sizeof (char),
 	      D_RELLEN (b, filenum, i), outputfile);
@@ -1416,15 +1467,15 @@ dotlines (FILE *outputfile, struct diff3_block *b, int filenum)
   return leading_dot;
 }
 
-/* Output to OUTPUTFILE a '.' line.  If LEADING_DOT is nonzero, also
+/* Output to OUTPUTFILE a '.' line.  If LEADING_DOT is true, also
    output a command that removes initial '.'s starting with line START
-   and continuing for NUM lines.  (START is long, not lin, for
+   and continuing for NUM lines.  (START is long int, not lin, for
    convenience with printf %ld formats.)  */
 
 static void
-undotlines (FILE *outputfile, bool leading_dot, long start, lin num)
+undotlines (FILE *outputfile, bool leading_dot, long int start, lin num)
 {
-  fprintf (outputfile, ".\n");
+  fputs (".\n", outputfile);
   if (leading_dot)
     {
       if (num == 1)
@@ -1459,7 +1510,8 @@ output_diff3_edscript (FILE *outputfile, struct diff3_block *diff,
 		       char const *file0, char const *file1, char const *file2)
 {
   bool leading_dot;
-  bool conflicts_found = 0, conflict;
+  bool conflicts_found = false;
+  bool conflict;
   struct diff3_block *b;
 
   for (b = reverse_diff3_blocklist (diff); b; b = b->next)
@@ -1470,14 +1522,14 @@ output_diff3_edscript (FILE *outputfile, struct diff3_block *diff,
 	   ? DIFF_ALL
 	   : DIFF_1ST + rev_mapping[b->correspond - DIFF_1ST]);
 
-      long low0, high0;
+      long int low0, high0;
 
       /* If we aren't supposed to do this output block, skip it.  */
       switch (type)
 	{
 	default: continue;
-	case DIFF_2ND: if (!show_2nd) continue; conflict = 1; break;
-	case DIFF_3RD: if (overlap_only) continue; conflict = 0; break;
+	case DIFF_2ND: if (!show_2nd) continue; conflict = true; break;
+	case DIFF_3RD: if (overlap_only) continue; conflict = false; break;
 	case DIFF_ALL: if (simple_only) continue; conflict = flagging; break;
 	}
 
@@ -1486,13 +1538,13 @@ output_diff3_edscript (FILE *outputfile, struct diff3_block *diff,
 
       if (conflict)
 	{
-	  conflicts_found = 1;
+	  conflicts_found = true;
 
 
 	  /* Mark end of conflict.  */
 
 	  fprintf (outputfile, "%lda\n", high0);
-	  leading_dot = 0;
+	  leading_dot = false;
 	  if (type == DIFF_ALL)
 	    {
 	      if (show_2nd)
@@ -1502,7 +1554,7 @@ output_diff3_edscript (FILE *outputfile, struct diff3_block *diff,
 		  leading_dot = dotlines (outputfile, b, mapping[FILE1]);
 		}
 	      /* Append lines from FILE2.  */
-	      fprintf (outputfile, "=======\n");
+	      fputs ("=======\n", outputfile);
 	      leading_dot |= dotlines (outputfile, b, mapping[FILE2]);
 	    }
 	  fprintf (outputfile, ">>>>>>> %s\n", file2);
@@ -1515,12 +1567,12 @@ output_diff3_edscript (FILE *outputfile, struct diff3_block *diff,
 
 	  fprintf (outputfile, "%lda\n<<<<<<< %s\n", low0 - 1,
 		   type == DIFF_ALL ? file0 : file1);
-	  leading_dot = 0;
+	  leading_dot = false;
 	  if (type == DIFF_2ND)
 	    {
 	      /* Prepend lines from FILE1.  */
 	      leading_dot = dotlines (outputfile, b, mapping[FILE1]);
-	      fprintf (outputfile, "=======\n");
+	      fputs ("=======\n", outputfile);
 	    }
 	  undotlines (outputfile, leading_dot, low0 + 1,
 		      D_NUMLINES (b, mapping[FILE1]));
@@ -1553,7 +1605,8 @@ output_diff3_edscript (FILE *outputfile, struct diff3_block *diff,
 		      low0, D_NUMLINES (b, mapping[FILE2]));
 	}
     }
-  if (finalwrite) fprintf (outputfile, "w\nq\n");
+  if (finalwrite)
+    fputs ("w\nq\n", outputfile);
   return conflicts_found;
 }
 
@@ -1575,7 +1628,8 @@ output_diff3_merge (FILE *infile, FILE *outputfile, struct diff3_block *diff,
 {
   int c;
   lin i;
-  bool conflicts_found = 0, conflict;
+  bool conflicts_found = false;
+  bool conflict;
   struct diff3_block *b;
   lin linesread = 0;
 
@@ -1592,8 +1646,8 @@ output_diff3_merge (FILE *infile, FILE *outputfile, struct diff3_block *diff,
       switch (type)
 	{
 	default: continue;
-	case DIFF_2ND: if (!show_2nd) continue; conflict = 1; break;
-	case DIFF_3RD: if (overlap_only) continue; conflict = 0; break;
+	case DIFF_2ND: if (!show_2nd) continue; conflict = true; break;
+	case DIFF_3RD: if (overlap_only) continue; conflict = false; break;
 	case DIFF_ALL: if (simple_only) continue; conflict = flagging;
 	  format_2nd = "||||||| %s\n";
 	  break;
@@ -1619,7 +1673,7 @@ output_diff3_merge (FILE *infile, FILE *outputfile, struct diff3_block *diff,
 
       if (conflict)
 	{
-	  conflicts_found = 1;
+	  conflicts_found = true;
 
 	  if (type == DIFF_ALL)
 	    {
@@ -1643,7 +1697,7 @@ output_diff3_merge (FILE *infile, FILE *outputfile, struct diff3_block *diff,
 			D_RELLEN (b, mapping[FILE1], i), outputfile);
 	    }
 
-	  fprintf (outputfile, "=======\n");
+	  fputs ("=======\n", outputfile);
 	}
 
       /* Put in lines from FILE2.  */
